import numpy as np
from noise import pnoise2

def perlin_map(
    width = 200,
    height = 200,
    scale = 100.0,
    octaves = 6,
    persistence = 0.5,
    lacunarity = 2.0,
) -> np.ndarray:
    """

    copied from noise.pnoise3:
    octaves -- specifies the number of passes for generating fBm noise,
    defaults to 1 (simple noise).

    persistence -- specifies the amplitude of each successive octave relative
    to the one below it. Defaults to 0.5 (each higher octave's amplitude
    is halved). Note the amplitude of the first pass is always 1.0.

    lacunarity -- specifies the frequency of each successive octave relative
    to the one below it, similar to persistence. Defaults to 2.0.

    repeatx, repeaty, repeatz -- specifies the interval along each axis when
    the noise values repeat. This can be used as the tile size for creating
    tileable textures

    base -- specifies a fixed offset for the input coordinates. Useful for
    generating different noise textures with the same repeat interval

    Parameters
    ----------
    width : int, optional
        _description_, by default 200
    height : int, optional
        _description_, by default 200
    scale : float, optional
        _description_, by default 100.0
    octaves : int, optional
        _description_, by default 6
    persistence : float, optional
        _description_, by default 0.5
    lacunarity : float, optional
        _description_, by default 2.0

    Returns
    -------
    np.ndarray
        _description_
    """
    terrain = np.zeros((height, width))

    for i in range(height):
        for j in range(width):
            terrain[i][j] = pnoise2(
                i / scale,
                j / scale,
                octaves=octaves,
                persistence=persistence,
                lacunarity=lacunarity,
                repeatx=width,
                repeaty=height,
                base=41
            )

    # Normalize terrain values to 0–1
    normalized_terrain = (terrain - terrain.min()) / (terrain.max() - terrain.min())

    return normalized_terrain



import numpy as np
from matplotlib.axes import Axes
from matplotlib.collections import PatchCollection
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.patches import Polygon

from scipy.spatial import Voronoi, voronoi_plot_2d

# from icecream import ic

def voronoi_map(
    axes: Axes,
    fig,
    heightmap: np.ndarray = None,
    colormap: LinearSegmentedColormap = None,
    config = None,
    ):
    if colormap is None:
        colormap = "terrain"

    np.random.seed(1234)
    if heightmap is not None:
        grid_shape = (heightmap.shape[0] - 1, heightmap.shape[1] - 1, )
    else:
        grid_shape = (200,200)

    # make up data points
    base_points = np.mgrid[
        0:(grid_shape[0]):(grid_shape[0]+1)*1j,
        0:(grid_shape[1]):(grid_shape[1]+1)*1j
    ]
    base_points = base_points.reshape(2,-1).T
    points = base_points + np.random.random_sample(base_points.shape) - .5

    # TODO 0.3: better dummy points or better yet exclude outermost ring of
    # points while keeping them saved as the basis of extending the map.
    # add 4 distant dummy points
    dummy_points = []
    dummy_points = [[999,999], [-999,999], [999,-999], [-999,-999]]
    base_points = np.append(base_points, dummy_points, axis = 0)
    points = np.append(points, dummy_points, axis = 0)

    # compute Voronoi tessellation
    voronoi = Voronoi(points)

    # voronoi_plot_2d(vor=voronoi, ax=axes, show_vertices=False, show_points=False)

    # map regions to points
    regions_to_points = np.argsort(voronoi.point_region)

    # Polygon patches for the voronoi regions
    polygons = []
    polygon_index = 0
    heights = np.zeros(len(points) - len(dummy_points))
    heightmap = heightmap.flatten()
    for i, region in enumerate(voronoi.regions):
        if not -1 in region and len(region) > 0:
            vertices = [voronoi.vertices[i] for i in region]
            polygon = Polygon(vertices, closed=True)
            polygons.append(polygon)
            # center = np.mean(polygon.get_xy(), axis=0)
            # axes.text(center[0], center[1], f"{i}")
            # index, = np.where(voronoi.point_region == i)
            # index = index[0]
            # ic(index, regions_to_points[i-1])
            heights[polygon_index] = heightmap[regions_to_points[i-1]]
            polygon_index += 1
            # axes.text(
            #     point[0], point[1],
            #     f"{j}, {i}",
            #     horizontalalignment='center',
            #     verticalalignment='center'
            # )
    # ic(points_in_regions[1:])
    # ic(region_to_polygon[1:])
    # ic(voronoi.point_region)
    # ic(regions_to_points)
    poly_collection = PatchCollection(patches=polygons, cmap=colormap)

    poly_collection.set_array(heights)
    axes.add_collection(poly_collection)
    fig.colorbar(poly_collection, ax=axes)

    # fix the range of axes
    axes.set_xlim([-1,grid_shape[0]+1])
    axes.set_ylim([-1,grid_shape[1]+1])



import matplotlib
import matplotlib.pyplot as plt
import numpy as np

from fimama.perlin import perlin_map
from fimama.voronoi import voronoi_map

matplotlib.use('qtagg')

def main() -> None:
    # plot
    fig, (ax1, ax2) = plt.subplots(nrows=1, ncols=2)
    ax1.set_aspect('equal', 'box')
    ax2.set_aspect('equal', 'box')

    width = 200
    height = 200
    terrain = perlin_map(width=width, height=height)
    # terrain = np.arange(width*height).reshape((width,height))

    # Read the colormap
    tmp = []
    for row in np.loadtxt( "geo-smooth.gpf" ):
        tmp.append( [ row[0], row[1:4] ] )
    colormap = matplotlib.colors.LinearSegmentedColormap.from_list( "geo-smooth", tmp )

    voronoi_map(fig=fig, axes=ax1, heightmap=terrain, colormap=colormap)

    # Colored elevation map
    ax2.imshow(X=terrain, cmap=colormap)
    plt.show()
